"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.ShadowPressable = void 0;
var _react = _interopRequireWildcard(require("react"));
var _reactNative = require("react-native");
var _reactNativeSkia = require("@shopify/react-native-skia");
var _reactNativeReanimated = _interopRequireWildcard(require("react-native-reanimated"));
var _constants = require("../constants.js");
var _utils = require("../utils.js");
var _jsxRuntime = require("react/jsx-runtime");
function _getRequireWildcardCache(e) { if ("function" != typeof WeakMap) return null; var r = new WeakMap(), t = new WeakMap(); return (_getRequireWildcardCache = function (e) { return e ? t : r; })(e); }
function _interopRequireWildcard(e, r) { if (!r && e && e.__esModule) return e; if (null === e || "object" != typeof e && "function" != typeof e) return { default: e }; var t = _getRequireWildcardCache(r); if (t && t.has(e)) return t.get(e); var n = { __proto__: null }, a = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var u in e) if ("default" !== u && {}.hasOwnProperty.call(e, u)) { var i = a ? Object.getOwnPropertyDescriptor(e, u) : null; i && (i.get || i.set) ? Object.defineProperty(n, u, i) : n[u] = e[u]; } return n.default = e, t && t.set(e, n), n; }
const PressButton = _reactNativeReanimated.default.createAnimatedComponent(_reactNative.Pressable);

/**
 * ShadowPressable
 * ----------------
 * A pressable component that casts a shadow when pressed.
 * The shadow effect is created using the `@shopify/react-native-skia` library.
 *
 * @param initialDepth - deprecated: set shadow depth using `shadowOffset` instead
 * @param shadowSpace - The space between the shadow and the component
 * @param duration - The duration of the shadow animation
 * @param damping - The damping factor of the shadow animation
 * @param isReflectedLightEnabled - Whether the reflected light effect is enabled
 */
const ShadowPressable = exports.ShadowPressable = /*#__PURE__*/(0, _react.memo)(function ShadowPressable({
  width = 0,
  height = 0,
  shadowSpace = _constants.SHADOW_SPACE,
  shadowBlur = _constants.SHADOW_BLUR,
  shadowColor = _constants.SHADOW_COLOR,
  reflectedLightColor = _constants.REFLECTED_LIGHT_COLOR,
  duration = _constants.DAMPING_DURATION,
  damping = _constants.DAMPING_RATIO,
  isReflectedLightEnabled = _constants.IS_REFLECTED_LIGHT_ENABLED,
  shadowOffset,
  reflectedLightOffset,
  reflectedLightBlur,
  style,
  backgroundColor,
  children,
  ...props
}) {
  const [boxSize, setBoxSize] = _react.default.useState({
    width,
    height
  });

  // Determine the final background color (pulling from `props.style` or a default).
  const _backgroundColor = (0, _utils.getBackgroundColor)({
    backgroundColor,
    style
  });
  const shadowProps = (0, _utils.getShadowProperty)({
    shadowOffset,
    shadowColor,
    shadowBlur,
    reflectedLightOffset,
    reflectedLightColor,
    reflectedLightBlur
  });
  const boxRadius = (0, _utils.numerify)(style?.borderRadius, 12);
  const depth = (0, _reactNativeReanimated.useSharedValue)(_constants.INITIAL_DEPTH);
  const offsetWidth = (0, _reactNativeReanimated.useDerivedValue)(() => (0, _reactNativeReanimated.interpolate)(depth.value, [-_constants.INITIAL_DEPTH, 0, _constants.INITIAL_DEPTH], [shadowProps.shadowOffset.width, 0, shadowProps.shadowOffset.width * damping]));
  const offsetHeight = (0, _reactNativeReanimated.useDerivedValue)(() => (0, _reactNativeReanimated.interpolate)(depth.value, [-_constants.INITIAL_DEPTH, 0, _constants.INITIAL_DEPTH], [shadowProps.shadowOffset.height, 0, shadowProps.shadowOffset.height * damping]));
  const rfOffsetWidth = (0, _reactNativeReanimated.useDerivedValue)(() => (0, _reactNativeReanimated.interpolate)(depth.value, [-_constants.INITIAL_DEPTH, 0, _constants.INITIAL_DEPTH], [shadowProps.reflectedLightOffset.width, 0, shadowProps.reflectedLightOffset.width * damping]));
  const rfOffsetHeight = (0, _reactNativeReanimated.useDerivedValue)(() => (0, _reactNativeReanimated.interpolate)(depth.value, [-_constants.INITIAL_DEPTH, 0, _constants.INITIAL_DEPTH], [shadowProps.reflectedLightOffset.height, 0, shadowProps.reflectedLightOffset.height * damping]));
  const inset = (0, _reactNativeReanimated.useDerivedValue)(() => depth.value <= 0);
  const blur = (0, _reactNativeReanimated.useDerivedValue)(() => (0, _reactNativeReanimated.interpolate)(depth.value, [-_constants.INITIAL_DEPTH, 0, _constants.INITIAL_DEPTH], [shadowProps.shadowBlur * 1.5, 0, shadowProps.shadowBlur]));
  const onPressIn = event => {
    depth.value = (0, _reactNativeReanimated.withTiming)(-_constants.INITIAL_DEPTH, {
      duration
    });
    props?.onPressIn?.(event);
  };
  const onPressOut = event => {
    depth.value = (0, _reactNativeReanimated.withTiming)(_constants.INITIAL_DEPTH, {
      duration
    });
    props?.onPressOut?.(event);
  };
  return /*#__PURE__*/(0, _jsxRuntime.jsxs)(PressButton, {
    ...props,
    onLayout: ({
      nativeEvent: {
        layout
      }
    }) => setBoxSize({
      width: layout.width,
      height: layout.height
    }),
    style: [style, _constants.COMMON_STYLES.canvasWrapper],
    onPressIn: onPressIn,
    onPressOut: onPressOut,
    children: [boxSize.width === 0 && boxSize.height === 0 ? null : /*#__PURE__*/(0, _jsxRuntime.jsx)(_reactNativeSkia.Canvas, {
      style: [style, _constants.COMMON_STYLES.canvas, {
        width: boxSize.width,
        height: boxSize.height
      }],
      children: /*#__PURE__*/(0, _jsxRuntime.jsxs)(_reactNativeSkia.RoundedRect, {
        x: shadowSpace,
        y: shadowSpace,
        width: boxSize.width - shadowSpace * 2,
        height: boxSize.height - shadowSpace * 2,
        r: boxRadius,
        color: _backgroundColor // The background fill of the rect
        ,
        children: [/*#__PURE__*/(0, _jsxRuntime.jsx)(_reactNativeSkia.Shadow, {
          dx: offsetWidth,
          dy: offsetHeight,
          blur: blur,
          color: shadowColor,
          inner: inset
        }), isReflectedLightEnabled && /*#__PURE__*/(0, _jsxRuntime.jsx)(_reactNativeSkia.Shadow, {
          dx: rfOffsetWidth,
          dy: rfOffsetHeight,
          blur: blur,
          color: reflectedLightColor,
          inner: true
        })]
      })
    }), children]
  });
});
//# sourceMappingURL=ShadowPressable.js.map