"use strict";

import React, { memo } from 'react';
import { View } from 'react-native';
import { getOuterShadowOffset, getBackgroundColor, getShadowProperty, isLinearProps } from "../utils.js";
import { COMMON_STYLES } from "../constants.js";
import { Canvas, Shadow } from '@shopify/react-native-skia';
import LinearGradientFill from "./ShadowLinearGradientFill.js";
import { CornerRadii } from "./CornerRadii.js";

/**
 * A unified interface for both "solid" (InnerShadow) and "linear" (LinearShadow).
 * We automatically detect "linear mode" by checking if the user provides
 * gradient props (colors, from, to, etc.).
 */
import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
const UnifiedShadowView = /*#__PURE__*/memo(function UnifiedShadowView({
  inset,
  isReflectedLightEnabled,
  width,
  height,
  style,
  backgroundColor,
  shadowOffset,
  shadowColor,
  shadowBlur,
  reflectedLightOffset,
  reflectedLightColor,
  reflectedLightBlur,
  children,
  ...props
}) {
  // Extract base fields
  const _backgroundColor = getBackgroundColor({
    backgroundColor,
    style
  });
  const shadowProps = getShadowProperty({
    shadowOffset,
    shadowColor,
    shadowBlur,
    reflectedLightOffset,
    reflectedLightColor,
    reflectedLightBlur
  });
  const isLinear = isLinearProps(props);

  // If isReflectedLightEnabled is undefined, default to `props.inset` (typical).
  const _isReflectedLightEnabled = isReflectedLightEnabled ?? inset;

  // Extract style-based width/height if present
  const styleWidth = width ?? (style && (Number.isNaN(Number(style.width)) ? 0 : Number(style.width)));
  const styleHeight = height ?? (style && (Number.isNaN(Number(style.height)) ? 0 : Number(style.height)));

  // Local state if we need to measure
  const [layoutSize, setLayoutSize] = React.useState({
    width: 0,
    height: 0
  });

  // Decide if we even need to attach onLayout
  const needMeasure = !styleWidth || !styleHeight;

  // onLayout only does something if we truly need measure
  const onLayout = React.useCallback(e => {
    if (!needMeasure) return;
    const {
      width: w,
      height: h
    } = e.nativeEvent.layout;
    if (w !== layoutSize.width || h !== layoutSize.height) {
      setLayoutSize({
        width: w,
        height: h
      });
    }
  }, [needMeasure, layoutSize]);

  // Final width/height
  const finalWidth = styleWidth ? styleWidth : layoutSize.width;
  const finalHeight = styleHeight ? styleHeight : layoutSize.height;

  // Create offset style for outer shadow if needed
  const outerShadowOffset = React.useMemo(() => getOuterShadowOffset({
    inset: !!inset,
    ...shadowProps
  }), [shadowProps, inset]);

  // Only show the canvas if we have a valid size
  const canRenderCanvas = finalWidth > 0 && finalHeight > 0;
  return /*#__PURE__*/_jsxs(View, {
    ...props,
    style: [style, COMMON_STYLES.canvasWrapper, outerShadowOffset // Possibly sets outer drop-shadow if needed
    ],
    onLayout: onLayout,
    children: [canRenderCanvas && /*#__PURE__*/_jsx(Canvas, {
      style: [COMMON_STYLES.canvas, {
        width: finalWidth,
        height: finalHeight
      }],
      children: /*#__PURE__*/_jsxs(CornerRadii, {
        width: finalWidth,
        height: finalHeight,
        style: style,
        backgroundColor: _backgroundColor,
        children: [isLinear && /*#__PURE__*/_jsx(LinearGradientFill, {
          ...props,
          // from, to, colors, etc.
          width: finalWidth,
          height: finalHeight
        }), inset && /*#__PURE__*/_jsx(Shadow, {
          dx: shadowProps.shadowOffset.width,
          dy: shadowProps.shadowOffset.height,
          blur: shadowProps.shadowBlur,
          color: shadowProps.shadowColor,
          inner: true
        }), _isReflectedLightEnabled && /*#__PURE__*/_jsx(Shadow, {
          dx: shadowProps.reflectedLightOffset.width,
          dy: shadowProps.reflectedLightOffset.height,
          blur: shadowProps.reflectedLightBlur,
          color: shadowProps.reflectedLightColor,
          inner: true
        })]
      })
    }), children]
  });
});

/**
 * ShadowView: for a basic “solid” background shadow
 * (no gradient props).
 */
export function ShadowView(props) {
  return /*#__PURE__*/_jsx(UnifiedShadowView, {
    ...props
  });
}

/**
 * LinearShadowView: for a linear gradient background shadow
 * (requires e.g. colors, from, to).
 */
export function LinearShadowView(props) {
  return /*#__PURE__*/_jsx(UnifiedShadowView, {
    ...props
  });
}
//# sourceMappingURL=ShadowView.js.map